#include "udp.h"

//#pragma config FPLLMUL = MUL_15, FPLLIDIV = DIV_4, FPLLODIV = DIV_1, FWDTEN = OFF		//60MHz
//#pragma config FPLLMUL = MUL_19, FPLLIDIV = DIV_4, FPLLODIV = DIV_1, FWDTEN = OFF		//76MHz
#pragma config FPLLMUL = MUL_20, FPLLIDIV = DIV_4, FPLLODIV = DIV_1, FWDTEN = OFF		//80MHz
//#pragma config FPLLMUL = MUL_24, FPLLIDIV = DIV_4, FPLLODIV = DIV_1, FWDTEN = OFF		//96MHz
//#pragma config FPLLMUL = MUL_20, FPLLIDIV = DIV_3, FPLLODIV = DIV_1, FWDTEN = OFF		//106MHz
//#pragma config FPLLMUL = MUL_21, FPLLIDIV = DIV_3, FPLLODIV = DIV_1, FWDTEN = OFF		//112MHz
//#pragma config FPLLMUL = MUL_24, FPLLIDIV = DIV_3, FPLLODIV = DIV_1, FWDTEN = OFF		//128MHz

#pragma config POSCMOD = XT, FNOSC = PRIPLL, FPBDIV = DIV_1

//Ethernet Alt I/O setting, RMII PHY
#pragma config FETHIO = OFF, FMIIEN = OFF

#define UART_BAUD					(115200)

u32	sys_freq;

void usart_init(void)
{
	int	pbClk;
   unsigned char data;

    UARTConfigure(UART1, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetFifoMode(UART1, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UART1, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART1, GetPeripheralClock(), UART_BAUD);
    UARTEnable(UART1, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));
}

void udp_init();
void udp_rcv_proc();
void arp_request(u16 *ipaddr);
void udp_trans();
void ts_init();
void ts_proc();

void bcas_init();
void bcas_proc();
void encrypt_multi2(u32 *emc, u32 *wky);

BOOL MACCheckLink(void);
BOOL MACIsLinked(void);
eEthLinkStat	_linkPrev;	// last value of the link status
void mk_udp_header();

IP_ADDR		TargetIPAddr;	// big-endian
MAC_ADDR	TargetMACAddr;	// big-endian

main()
{
	u32 last_link_chk = 0;
	u32 last_target_arp = 0;
	u32 pb_clk;
	const u8 pll_mul[8]={15,16,17,18,19,20,21,24};
	const u8 pll_div[8]={1,2,3,4,5,6,10,12};
	
	// disable JTAG port
	DDPCONbits.JTAGEN = 0;
    
	sys_freq = (pll_mul[DEVCFG2bits.FPLLMUL] * 16000000 / pll_div[DEVCFG2bits.FPLLIDIV]) / (1 << DEVCFG2bits.FPLLODIV);

	pb_clk = SYSTEMConfig(sys_freq, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE | SYS_CFG_PB_BUS);
	//BMXCONbits.BMXWSDRM = 1;	//CPU Data RAM access wait = 1

	mBMXSetArbMode(0);		//CPUD
	//mBMXSetArbMode(1);	//DMAD
   	DmaEnable(1);

	// Turn on the interrupts
	INTEnableSystemMultiVectoredInt();

   	usart_init();
   	printf("SYSCLK = %d PBCLK = %d\n", sys_freq, pb_clk);
	WriteCoreTimer(0);
   	   	
	udp_init();
	ts_init();
	bcas_init();
	
	for(;;){
		u32 ctimer;
		
		ctimer = ReadCoreTimer();
		
		if(*((u32 *)&TargetMACAddr) != 0){
			ts_proc();
		}	

		if(U3ASTAbits.URXDA){
			bcas_proc();
		}
		
		if(ctimer - last_link_chk > 500*CTIMER_MS){
			MACCheckLink();
			last_link_chk = ctimer;
		}
		else if((_linkPrev & ETH_LINK_ST_UP) != 0){
			//NĂ
			if(!(ETHSTATbits.TXBUSY)){
				//M\
				if((*((u32 *)&TargetMACAddr) == 0) && (ctimer - last_target_arp > 1000*CTIMER_MS)){
					arp_request((u16 *)&TargetIPAddr);
					last_target_arp = ctimer;
				}
				else if(ETHIRQbits.RXDONE){
					udp_rcv_proc();
					ETHIRQCLR = ETH_EV_RXDONE;
				}
				else if(*((u32 *)&TargetMACAddr) != 0){
					udp_trans();
				}
			}
		}
	}
	printf("Program END.\n");
	assert(0);
} // main
